#if !defined  HAVE_PERM_REV2_H__
#define       HAVE_PERM_REV2_H__


#include "perm/reverse.h"
#include "comb/comb-print.h"
#include "fxttypes.h"


#define PERM_REV2_FIXARRAYS // use arrays instead of pointers (speedup)

class perm_rev2
// Permutations by reversing prefixes, CAT algorithm.
// Optimized version for permutations of at least three elements.
{
public:
#ifndef PERM_REV2_FIXARRAYS
    ulong *d_;  // mixed radix digits with radix = [2, 3, 4, ..., n-1, (sentinel=-1)]
    ulong *p_;  // permutation
#else
    ulong d_[32];
    ulong p_[32];
#endif
    ulong n_;   // permutations of n elements
    ulong ct_;  // count 5,4,3,2,1,(0); nonzero ==> easy cases

public:
    perm_rev2(ulong n)  // must have n>=3
    {
        n_ = n;
#ifndef PERM_REV2_FIXARRAYS
        p_ = new ulong[n_];
        d_ = new ulong[n_];
#endif
        d_[n-1] = -1UL;  // sentinel
        first();
    }

    ~perm_rev2()
    {
#ifndef PERM_REV2_FIXARRAYS
        delete [] p_;
        delete [] d_;
#endif
    }

    const ulong * data()  const { return p_; }

    void first()
    {
        for (ulong k=0; k<n_-1; ++k)  d_[k] = 0;
        for (ulong k=0; k<n_; ++k)  p_[k] = k;
        ct_ = 5;
    }

    bool next()
    {
        if ( ct_!=0 )  // easy case(s)
        {
            --ct_;
            swap2(p_[0], p_[1 + (ct_ & 1)]);
            return true;
        }
        else  // increment mixed radix number:
        {
            ct_ = 5;  // reset counter

            ulong j = 2;  // note: start with 2
            while ( d_[j]==j+1 )  { d_[j]=0; ++j; }

            // j==n-1 for last permutation
            if ( j!=n_-1 ) // only if no overflow
            {
                ++d_[j];
                reverse(p_, j+2); // update permutation
                return true;
            }
            else  return false;
        }
    }

    void print(const char *bla, bool dfz=false)  const
    // If dfz is true then Dots are printed For Zeros.
    { print_perm(bla, data(), n_, dfz); }
};
// -------------------------

//#undef PERM_REV2_FIXARRAYS


#endif  // !defined HAVE_PERM_REV2_H__
