#if !defined  HAVE_SUBSET_DEBRUIJN_H__
#define       HAVE_SUBSET_DEBRUIJN_H__


#include "fxttypes.h"
#include "comb/debruijn.h"
#include "aux1/shift.h"


class subset_debruijn : public debruijn
// Subsets of the set {0,1,2,...,n-1} in an order
// determined by a De Bruijn sequence.
// Note: work per subset proportional to n.
{
protected:
    ulong *x;   // subset as delta set
    ulong el_;  // new element to be shifted in
    ulong num_; // number of elements in subset

public:
    subset_debruijn(ulong n)
        : debruijn(2, n)
    {
        x = new ulong[n_];
        first();
    }

    ~subset_debruijn()  { delete [] x; }


    void first()
    {
        for (ulong j=0; j<n_; ++j)  x[j] = 1; //fill(x, n_, 1UL);  // sequence end = 1111
        num_ = n_;

//        el_ = 0;
        debruijn::first();  // use binary shift register sequence

//        // forward to empty set:
//        while ( next() )  { ; }
    }


    ulong next()  // return number of elements
    {
        el_ = debruijn::next();
        if ( debruijn::done() )  // allow wrap around
        {
            debruijn::first();
            el_ = 0;
        }

        sync_x();
        return  num_;
    }

    const ulong * data()  const  { return x; }


protected:
    void sync_x()
    // NOTE: performance bottleneck
    // might want to use "sliding window"
    {
        num_ -= x[n_-1];
        shift_right(x, n_, 1);
        x[0] = el_;
        num_ += el_;
    }

private:
    int done()  const;  // forbidden (exists in base class)
};
// -------------------------



#endif  // !defined HAVE_SUBSET_DEBRUIJN_H__
