#if !defined  HAVE_PRIORITYQUEUE_H__
#define       HAVE_PRIORITYQUEUE_H__


#include "fxttypes.h"
#include "realloc.h"

//#include "aux1/copy.h"
//#define copy fxtaux::copy // avoid unwanted std::copy

#include "aux0/swap.h"


//<<
#if  1
// next() is the one with the smallest key
// i.e.  extract_next()  is  extract_min()
#define _CMP_ <
#define _CMPEQ_ <=
#else
// next() is the one with the biggest key
// i.e.  extract_next()  is  extract_max()
#define _CMP_ >
#define _CMPEQ_ >=
#endif
//>>

template <typename Type1, typename Type2>
class priority_queue
// Priority queue.
// Can grow dynamically.
{
public:
    Type1 *t1_;  // time:   t1[1..s]  one-based array!
    Type2 *e1_;  // events: e1[1..s]  one-based array!
    ulong s_;    // allocated size (# of elements)
    ulong n_;    // current number of events
    ulong gq_;   // grow gq elements if necessary, 0 for "never grow"


public:
    priority_queue(ulong n, ulong growq=0)
    {
        s_ = n;
        t1_ = new Type1[s_] - 1;
        e1_ = new Type2[s_] - 1;

        n_ = 0;
        gq_ = growq;
    }

    ~priority_queue()
    {
        delete [] (t1_+1);
        delete [] (e1_+1);
    }

    ulong num()  const  { return n_; }

    Type1 get_next_t()  const  { return t1_[1]; }
    Type2 get_next_e()  const  { return e1_[1]; }

    Type1 get_next(Type2 &e)  const
    // No check if empty
    {
        e = e1_[1];
        return t1_[1];
    }

    Type1 extract_next(Type2 &e)
    {
        Type1 m = get_next(e);
        if ( 0 != n_ )
        {
            t1_[1] = t1_[n_];  e1_[1] = e1_[n_];
            --n_;
            heapify1(1);
        }
        // else error
        return m;
    }

    bool insert(const Type1 &t, const Type2 &e)
    // Insert event e at time t
    // Return true if successful
    // else (i.e. space exhausted and 0==gq_) false
    {
        if ( n_ >= s_ )
        {
            if ( 0==gq_ )  return false;  // growing disabled
            grow();
        }

        ++n_;
        ulong j = n_;
        while ( j > 1 )
        {
            ulong k = (j>>1);  // k==parent(j)
            if ( t1_[k] _CMPEQ_ t )  break;
            t1_[j] = t1_[k];  e1_[j] = e1_[k];
            j = k;
        }
        t1_[j] = t;
        e1_[j] = e;

        return true;
    }

    void reschedule_next(Type1 t)
    {
        t1_[1] = t;
        heapify1(1);
    }


private:
    void heapify1(ulong k)
    {
        ulong m = k;

    hstart:
        ulong l = (k<<1); // left(k);
        ulong r = l + 1;  // right(k);
        if ( (l <= n_) && (t1_[l] _CMP_ t1_[k]) )  m = l;
        if ( (r <= n_) && (t1_[r] _CMP_ t1_[m]) )  m = r;

        if ( m != k )
        {
            swap2(t1_[k], t1_[m]);  swap2(e1_[k], e1_[m]);
//            heapify1(m);
            k = m;
            goto hstart;  // tail recursion
        }
    }

    void grow()
    {
        ulong ns = s_ + gq_;  // new size
        t1_ = ReAlloc<Type1>(t1_+1, ns, s_) - 1;
        e1_ = ReAlloc<Type2>(e1_+1, ns, s_) - 1;
        s_ = ns;
    }
};
// -------------------------

#undef _CMP_
#undef _CMPEQ_

//#undef copy // end (avoid unwanted std::copy)

#endif  // !defined HAVE_PRIORITYQUEUE_H__
